/*******************************************************************************
* File Name: OPAmp.c
* Version 1.80
*
* Description:
*  This file provides the source code to the API for OpAmp (Analog Buffer) 
*  Component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "OPAmp.h"
#include <CyLib.h>

uint8 OPAmp_initVar = 0u;


/*******************************************************************************   
* Function Name: OPAmp_Init
********************************************************************************
*
* Summary:
*  Initialize component's parameters to the parameters set by user in the 
*  customizer of the component placed onto schematic. Usually called in 
*  OPAmp_Start().
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void OPAmp_Init(void) 
{
    OPAmp_SetPower(OPAmp_DEFAULT_POWER);
}


/*******************************************************************************   
* Function Name: OPAmp_Enable
********************************************************************************
*
* Summary:
*  Enables the OpAmp block operation
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void OPAmp_Enable(void) 
{
    /* Enable negative charge pumps in ANIF */
    OPAmp_PUMP_CR1_REG  |= (OPAmp_PUMP_CR1_CLKSEL | OPAmp_PUMP_CR1_FORCE);

    /* Enable power to buffer in active mode */
    OPAmp_PM_ACT_CFG_REG |= OPAmp_ACT_PWR_EN;

    /* Enable power to buffer in alternative active mode */
    OPAmp_PM_STBY_CFG_REG |= OPAmp_STBY_PWR_EN;
}


/*******************************************************************************
* Function Name:   OPAmp_Start
********************************************************************************
*
* Summary:
*  The start function initializes the Analog Buffer with the default values and 
*  sets the power to the given level. A power level of 0, is same as 
*  executing the stop function.
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  OPAmp_initVar: Used to check the initial configuration, modified 
*  when this function is called for the first time.
*
*******************************************************************************/
void OPAmp_Start(void) 
{
    if(OPAmp_initVar == 0u)
    {
        OPAmp_initVar = 1u;
        OPAmp_Init();
    }

    OPAmp_Enable();
}


/*******************************************************************************
* Function Name: OPAmp_Stop
********************************************************************************
*
* Summary:
*  Powers down amplifier to lowest power state.
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void OPAmp_Stop(void) 
{
    /* Disable negative charge pumps for ANIF only if one ABuf is turned ON */
    if(OPAmp_PM_ACT_CFG_REG == OPAmp_ACT_PWR_EN)
    {
        OPAmp_PUMP_CR1_REG &= ~(OPAmp_PUMP_CR1_CLKSEL | OPAmp_PUMP_CR1_FORCE);
    }
    
    /* Disable power to buffer in active mode template */
    OPAmp_PM_ACT_CFG_REG &= ~OPAmp_ACT_PWR_EN;

    /* Disable power to buffer in alternative active mode template */
    OPAmp_PM_STBY_CFG_REG &= ~OPAmp_STBY_PWR_EN;
}


/*******************************************************************************
* Function Name: OPAmp_SetPower
********************************************************************************
*
* Summary:
*  Sets power level of Analog buffer.
*
* Parameters: 
*  power: PSoC3: Sets power level between low (1) and high power (3).
*         PSoC5: Sets power level High (0)
*
* Return:
*  void
*
**********************************************************************************/
void OPAmp_SetPower(uint8 power) 
{
    /* Only High power can be used in PSoC5 */
    #if CY_PSOC5A
        CYASSERT(power == OPAmp_HIGHPOWER);
    #endif   /* (CY_PSOC5A) */

    OPAmp_CR_REG = ((OPAmp_CR_REG & ~OPAmp_PWR_MASK) | 
                               ( power & OPAmp_PWR_MASK));   /* Set device power */
}


/* [] END OF FILE */
